@@ -22,6 +22,7 @@ import re, binascii
 import os, stat, os.path, mimetypes, urllib, time
 from pygopherd import handlers, protocols, GopherExceptions
 from pygopherd.protocols.base import BaseGopherProtocol
+import pygopherd.version
 import cgi
 
 class HTTPProtocol(BaseGopherProtocol):
@@ -124,14 +125,25 @@ class HTTPProtocol(BaseGopherProtocol):
         retstr += "\n<HTML><HEAD><TITLE>Gopher"
         if self.entry.getname():
             retstr += ": " + cgi.escape(self.entry.getname())
-        retstr += "</TITLE></HEAD><BODY><H1>Gopher"
+        retstr += "</TITLE></HEAD><BODY>"
+        if self.config.has_option("protocols.http.HTTPProtocol", "pagetopper"):
+            retstr += re.sub('GOPHERURL',
+                             self.entry.geturl(self.server.server_name,
+                                               self.server.server_port),
+                             self.config.get("protocols.http.HTTPProtocol",
+                                             "pagetopper"))
+        retstr += "<H1>Gopher"
         if self.entry.getname():
             retstr += ": " + cgi.escape(self.entry.getname())
         retstr += '</H1><TABLE WIDTH="100%" CELLSPACING="1" CELLPADDING="0">'
         return retstr
 
     def renderdirend(self, entry):
-        return '</TABLE></BODY></HTML>'
+        retstr = "</TABLE><HR>\n[<A HREF=\"/\">server top</A>]"
+        retstr += " [<A HREF=\"%s\">view with gopher</A>]" % entry.geturl()
+        retstr += '<BR>Generated by <A HREF="%s">%s</A>' % (
+            pygopherd.version.homepage, pygopherd.version.productname)
+        return retstr + "\n</BODY></HTML>\n"
 
     def filenotfound(self, msg):
         self.wfile.write("HTTP/1.0 404 Not Found\n")
